# Generated by Django 4.2.5 on 2023-10-02 05:36

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0187_auto_20230926_1627'),
    ]

    operations = [
        migrations.AlterField(
            model_name='learnercreditenterprisecourseenrollment',
            name='enterprise_course_enrollment',
            field=models.OneToOneField(blank=True, help_text='The course enrollment the associated subsidy is for.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='%(class)s_enrollment_fulfillment', to='enterprise.enterprisecourseenrollment'),
        ),
        migrations.AlterField(
            model_name='learnercreditenterprisecourseenrollment',
            name='enterprise_course_entitlement',
            field=models.OneToOneField(blank=True, help_text='The course entitlement the associated subsidy is for.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='%(class)s_entitlement_fulfillment', to='enterprise.enterprisecourseentitlement'),
        ),
        migrations.AlterField(
            model_name='licensedenterprisecourseenrollment',
            name='enterprise_course_enrollment',
            field=models.OneToOneField(blank=True, help_text='The course enrollment the associated subsidy is for.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='%(class)s_enrollment_fulfillment', to='enterprise.enterprisecourseenrollment'),
        ),
        migrations.AlterField(
            model_name='licensedenterprisecourseenrollment',
            name='enterprise_course_entitlement',
            field=models.OneToOneField(blank=True, help_text='The course entitlement the associated subsidy is for.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='%(class)s_entitlement_fulfillment', to='enterprise.enterprisecourseentitlement'),
        ),
    ]



"""
URL definitions for integrated_channels/canvas API version 1.
"""

from rest_framework import routers

from .views import CanvasConfigurationViewSet

app_name = 'canvas'
router = routers.DefaultRouter()
router.register(r'configuration', CanvasConfigurationViewSet, basename="configuration")
urlpatterns = router.urls



"""
Transmits consenting enterprise learner data to the integrated channels.
"""

from django.contrib import auth
from django.core.management.base import BaseCommand, CommandError
from django.utils.translation import gettext as _

from integrated_channels.integrated_channel.management.commands import IntegratedChannelCommandMixin
from integrated_channels.integrated_channel.tasks import transmit_subsection_learner_data

User = auth.get_user_model()


class Command(IntegratedChannelCommandMixin, BaseCommand):
    """
    Management command which transmits learner course completion data to the IntegratedChannel(s) configured for the
    given EnterpriseCustomer.

    Collect the enterprise learner data for enrollments with data sharing consent, and transmit each to the
    EnterpriseCustomer's configured IntegratedChannel(s).
    """
    help = _('''
    Transmit Enterprise learner assessment level reporting data for the given EnterpriseCustomer.
    ''')
    stealth_options = ('enterprise_customer_slug', 'user1', 'user2')

    def add_arguments(self, parser):
        """
        Add required --api_user argument to the parser.
        """
        parser.add_argument(
            '--api_user',
            dest='api_user',
            required=True,
            metavar='LMS_API_USERNAME',
            help=_('Username of a user authorized to fetch grades from the LMS API.'),
        )
        super().add_arguments(parser)

    def handle(self, *args, **options):
        """
        Transmit the assessment level learner data for the EnterpriseCustomer(s) to the active integration channels.
        """
        # Ensure that we were given an api_user name, and that User exists.
        api_username = options['api_user']
        options['assessment_level_support'] = True
        try:
            User.objects.get(username=api_username)
        except User.DoesNotExist as no_user_error:
            raise CommandError(
                _('A user with the username {username} was not found.').format(username=api_username)
            ) from no_user_error

        # Transmit the learner data to each integrated channel
        for integrated_channel in self.get_integrated_channels(options):
            transmit_subsection_learner_data.delay(
                api_username,
                integrated_channel.channel_code(),
                integrated_channel.pk
            )



# Generated by Django 3.2.23 on 2024-04-08 13:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0205_auto_20240408_1117'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisegroupmembership',
            name='removed_at',
            field=models.DateTimeField(blank=True, default=None, help_text='The moment at which the membership record was revoked by an Enterprise admin.', null=True),
        ),
        migrations.AddField(
            model_name='enterprisegroupmembership',
            name='status',
            field=models.CharField(blank=True, choices=[('removed', 'Removed'), ('accepted', 'Accepted'), ('pending', 'Pending')], default='pending', help_text='Current status of the membership record', max_length=20, null=True, verbose_name='Membership Status'),
        ),
        migrations.AddField(
            model_name='historicalenterprisegroupmembership',
            name='removed_at',
            field=models.DateTimeField(blank=True, default=None, help_text='The moment at which the membership record was revoked by an Enterprise admin.', null=True),
        ),
        migrations.AddField(
            model_name='historicalenterprisegroupmembership',
            name='status',
            field=models.CharField(blank=True, choices=[('removed', 'Removed'), ('accepted', 'Accepted'), ('pending', 'Pending')], default='pending', help_text='Current status of the membership record', max_length=20, null=True, verbose_name='Membership Status'),
        ),
    ]



# -*- coding: utf-8 -*-
"""
Learner data exporter for Enterprise Integrated Channel Degreed.
"""

from datetime import datetime
from logging import getLogger

from django.apps import apps

from integrated_channels.catalog_service_utils import get_course_id_for_enrollment
from integrated_channels.integrated_channel.exporters.learner_data import LearnerExporter
from integrated_channels.utils import generate_formatted_log

LOGGER = getLogger(__name__)


class Degreed2LearnerExporter(LearnerExporter):
    """
    Class to provide a Degreed learner data transmission audit prepared for serialization.
    """

    def get_learner_data_records(
            self,
            enterprise_enrollment,
            completed_date=None,
            course_completed=False,
            **kwargs
    ):  # pylint: disable=arguments-differ
        """
        Return a Degreed2LearnerDataTransmissionAudit with the given enrollment and course completion data.

        If no remote ID can be found, return None.
        """
        percent_grade = kwargs.get('grade_percent') * 100 if kwargs.get('grade_percent') else None
        # Degreed expects completion dates of the form 'yyyy-mm-ddTHH:MM:SS'.
        degreed_completed_timestamp = completed_date.strftime('%Y-%m-%dT%H:%M:%S') if isinstance(
            completed_date, datetime
        ) else None
        try:
            remote_id = enterprise_enrollment.enterprise_customer_user.get_remote_id(
                self.enterprise_configuration.idp_id
            )
        except Exception as e:  # pylint: disable=broad-except
            LOGGER.error(generate_formatted_log(
                self.enterprise_configuration.channel_code(),
                enterprise_enrollment.enterprise_customer_user.enterprise_customer.uuid,
                enterprise_enrollment.enterprise_customer_user.user_id,
                enterprise_enrollment.course_id,
                '[Degreed2Client] get_learner_data_records failed, possibly due to an invalid customer configuration. '
                f'Error: {e}'
            ))
            return None

        if remote_id is not None:
            Degreed2LearnerDataTransmissionAudit = apps.get_model(
                'degreed2',
                'Degreed2LearnerDataTransmissionAudit'
            )
            course_id = get_course_id_for_enrollment(enterprise_enrollment)
            # We only want to send one record per enrollment and course, so we check if one exists first.
            learner_transmission_record = Degreed2LearnerDataTransmissionAudit.objects.filter(
                enterprise_course_enrollment_id=enterprise_enrollment.id,
                course_id=course_id,
            ).first()
            if learner_transmission_record is None:
                learner_transmission_record = Degreed2LearnerDataTransmissionAudit(
                    enterprise_course_enrollment_id=enterprise_enrollment.id,
                    degreed_user_email=enterprise_enrollment.enterprise_customer_user.user_email,
                    user_email=enterprise_enrollment.enterprise_customer_user.user_email,
                    course_id=get_course_id_for_enrollment(enterprise_enrollment),
                    completed_timestamp=completed_date,
                    degreed_completed_timestamp=degreed_completed_timestamp,
                    course_completed=course_completed,
                    grade=percent_grade,
                    enterprise_customer_uuid=enterprise_enrollment.enterprise_customer_user.enterprise_customer.uuid,
                    plugin_configuration_id=self.enterprise_configuration.id,
                )
            # We return one record here, with the course key, that was sent to the integrated channel.
            return [learner_transmission_record]
        LOGGER.info(generate_formatted_log(
            self.enterprise_configuration.channel_code(),
            enterprise_enrollment.enterprise_customer_user.enterprise_customer.uuid,
            enterprise_enrollment.enterprise_customer_user.user_id,
            enterprise_enrollment.course_id,
            ('get_learner_data_records finished. No learner data was sent for this LMS User Id because '
             'Degreed2 User ID not found for [{name}]'.format(
                 name=enterprise_enrollment.enterprise_customer_user.enterprise_customer.name
             ))))
        return None



"""
Middleware for enterprise app.
"""

from django.conf import settings
from django.utils.deprecation import MiddlewareMixin

from enterprise.utils import get_enterprise_customer_for_user

try:
    from openedx.core.djangoapps.lang_pref import LANGUAGE_KEY
except ImportError:
    LANGUAGE_KEY = None

try:
    from openedx.core.djangoapps.user_api.errors import UserAPIInternalError, UserAPIRequestError
except ImportError:
    UserAPIInternalError = None
    UserAPIRequestError = None

try:
    from openedx.core.djangoapps.user_api.preferences.api import get_user_preference
except ImportError:
    get_user_preference = None

try:
    from openedx.core.lib.mobile_utils import is_request_from_mobile_app
except ImportError:
    is_request_from_mobile_app = None


class EnterpriseLanguagePreferenceMiddleware(MiddlewareMixin):
    """
    Middleware for enterprise language preference.

    Ensures that, once set, a user's preferences are reflected in the page
    whenever they are logged in.
    """

    def process_request(self, request):
        """
        Perform the following checks
            1. Check that the user is authenticated and belongs to an enterprise customer.
            2. Check that the enterprise customer has a language set via the `default_language` column on
                EnterpriseCustomer model.
            3. Check that user has not set a language via its account settings page.

        If all the above checks are satisfied then set request._anonymous_user_cookie_lang to the `default_language` of
        EnterpriseCustomer model instance. This attribute will later be used by the `LanguagePreferenceMiddleware`
        middleware for setting the user preference. Since, this middleware relies on `LanguagePreferenceMiddleware`
        so it must always be followed by `LanguagePreferenceMiddleware`. Otherwise, it will not work.
        """
        # If the user is logged in, check for their language preference and user's enterprise configuration.
        # Also check for real user, if current user is a masquerading user.
        user_pref, current_user = None, None
        if hasattr(request, 'user'):
            current_user = getattr(request.user, 'real_user', request.user)

        if current_user and current_user.is_authenticated:
            enterprise_customer = get_enterprise_customer_for_user(current_user)

            if enterprise_customer and enterprise_customer.default_language:
                # Get the user's language preference
                try:
                    user_pref = get_user_preference(current_user, LANGUAGE_KEY)
                except (UserAPIRequestError, UserAPIInternalError):
                    # Ignore errors related to user preferences not found.
                    pass

                # This is to handle a bug where a user is logged in multiple tabs/browsers.
                # User updates the language preference in one tab. After update, user's language preference
                # in the database and language cookie in the browser is set to new language.
                # But in the other tab/browser, user still has the old language cookie. So when the user
                # refreshes the page, the language cookie is sent to the server and overrides the user's preference.
                # NOTE: The assumption here is to consider user's language preference as the single source of truth.
                cookie_lang = request.COOKIES.get(settings.LANGUAGE_COOKIE_NAME)
                if user_pref and cookie_lang and cookie_lang != user_pref:
                    # pylint: disable=protected-access
                    request._anonymous_user_cookie_lang = user_pref
                    request.COOKIES[settings.LANGUAGE_COOKIE_NAME] = user_pref

                # If user's language preference is not set and enterprise customer has a default language configured
                # then set the default language as the learner's language
                if not user_pref and not is_request_from_mobile_app(request):
                    # pylint: disable=protected-access
                    request._anonymous_user_cookie_lang = enterprise_customer.default_language
                    request.COOKIES[settings.LANGUAGE_COOKIE_NAME] = enterprise_customer.default_language



"""
Content metadata exporter for Canvas
"""

from datetime import datetime
from logging import getLogger

from integrated_channels.integrated_channel.exporters.content_metadata import ContentMetadataExporter

LOGGER = getLogger(__name__)


def convert_date_str(date_str):
    '''
    Returns formatted date string from ISO8601 format (e.g. 2011-01-01T01:00:10Z)
    to a human readable suitable for use in Canvas
    Return 'N/A' if input arg is None, or 'N/A'
    If format is not ISO8601, returns original string.
    '''
    if not date_str or date_str == 'N/A':
        return date_str
    try:
        start_date = datetime.strptime(date_str, '%Y-%m-%dT%H:%M:%SZ')
        formatted_start_date = start_date.strftime('%a %b %d %Y %H:%M:%S')
    except ValueError:
        return date_str
    return formatted_start_date


class CanvasContentMetadataExporter(ContentMetadataExporter):
    """
        Canvas implementation of ContentMetadataExporter.
    """
    DATA_TRANSFORM_MAPPING = {
        'name': 'title',
        'start_at': 'start',
        'end_at': 'end',
        'integration_id': 'key',
        'syllabus_body': 'description',
        'default_view': 'default_view',
        'image_url': 'image_url',
        'is_public': 'is_public',
        'self_enrollment': 'self_enrollment',
        'course_code': 'key',
        'indexed': 'indexed',
        'restrict_enrollments_to_course_dates': 'restrict_enrollments_to_course_dates',
    }
    SKIP_KEY_IF_NONE = True

    LONG_STRING_LIMIT = 2000

    def transform_restrict_enrollments_to_course_dates(self, content_metadata_item):  # pylint: disable=unused-argument
        '''
        This enforces the course to use the participation type of 'Course' rather than Term
        '''
        return True

    def transform_description(self, content_metadata_item):
        """
        Return the course description and enrollment url as Canvas' syllabus body attribute.
        This will display in the Syllabus tab in Canvas.
        """
        enrollment_url = content_metadata_item.get('enrollment_url', None)
        base_description = "<a href={enrollment_url}>Go to edX course page</a><br />".format(
            enrollment_url=enrollment_url)
        full_description = content_metadata_item.get('full_description') or None
        if full_description and len(full_description + enrollment_url) <= self.LONG_STRING_LIMIT:
            description = "{base_description}{full_description}".format(
                base_description=base_description,
                full_description=full_description
            )
        else:
            short_description = content_metadata_item.get(
                'short_description', content_metadata_item.get('title', '')
            )
            description = "{base_description}{short_description}".format(
                base_description=base_description, short_description=short_description
            )

        formatted_start_date = convert_date_str(content_metadata_item.get('start', 'N/A'))
        formatted_end_date = convert_date_str(content_metadata_item.get('end', 'N/A'))
        description = (f"{description} <br />"
                       f"<br />Starts: {formatted_start_date}"
                       f"<br />Ends: {formatted_end_date}")

        return description

    def transform_default_view(self, content_metadata_item):  # pylint: disable=unused-argument
        """
        Sets the Home page view in Canvas. We're using Syllabus.
        """
        return 'syllabus'

    def transform_is_public(self, content_metadata_item):  # pylint: disable=unused-argument
        """
        Whether to make the course content visible to students by default. Note that setting this feature to false
        will not remove the course from discovery, but will rather remove all visible content from the course if
        any student attempts to access it.
        """
        return True

    def transform_self_enrollment(self, content_metadata_item):  # pylint: disable=unused-argument
        """
        Whether to allow students to self enroll. Helps students enroll via link or enroll button.
        """
        return True

    def transform_indexed(self, content_metadata_item):  # pylint: disable=unused-argument
        """
        Whether to make the course default to the public index or not.
        """
        return 1

    def transform_start(self, content_metadata_item):
        """
        Returns none if no start date is available (the case for courses),
        and the start date (the case for course run data)
        """
        return content_metadata_item.get('start', None)

    def transform_end(self, content_metadata_item):
        """
        Returns none if no end date is available (the case for courses),
         and the end date (the case for course run data)
        """
        return content_metadata_item.get('end', None)



# -*- coding: utf-8 -*-
"""
Django admin integration for configuring degreed app to communicate with Degreed systems.
"""

from django_object_actions import DjangoObjectActions

from django.contrib import admin, messages
from django.core.exceptions import ValidationError
from django.http import HttpResponseRedirect

from integrated_channels.degreed2.models import (
    Degreed2EnterpriseCustomerConfiguration,
    Degreed2LearnerDataTransmissionAudit,
)
from integrated_channels.integrated_channel.admin import BaseLearnerDataTransmissionAuditAdmin


@admin.register(Degreed2EnterpriseCustomerConfiguration)
class Degreed2EnterpriseCustomerConfigurationAdmin(DjangoObjectActions, admin.ModelAdmin):
    """
    Django admin model for Degreed2EnterpriseCustomerConfiguration.
    """

    list_display = (
        "enterprise_customer_name",
        "active",
        "degreed_base_url",
        "degreed_token_fetch_base_url",
        "modified",
    )

    readonly_fields = (
        "enterprise_customer_name",
        "transmission_chunk_size",
    )

    raw_id_fields = (
        "enterprise_customer",
    )

    list_filter = ("active",)
    search_fields = ("enterprise_customer_name",)
    change_actions = ("force_content_metadata_transmission",)

    class Meta:
        model = Degreed2EnterpriseCustomerConfiguration

    def enterprise_customer_name(self, obj):
        """
        Returns: the name for the attached EnterpriseCustomer.

        Args:
            obj: The instance of Degreed2EnterpriseCustomerConfiguration
                being rendered with this admin form.
        """
        return obj.enterprise_customer.name

    @admin.action(
        description="Force content metadata transmission for this Enterprise Customer"
    )
    def force_content_metadata_transmission(self, request, obj):
        """
        Updates the modified time of the customer record to retransmit courses metadata
        and redirects to configuration view with success or error message.
        """
        try:
            obj.enterprise_customer.save()
            messages.success(
                request,
                f'''The degreed2 enterprise customer content metadata
                “<Degreed2EnterpriseCustomerConfiguration for Enterprise
                {obj.enterprise_customer.name}>” was updated successfully.''',
            )
        except ValidationError:
            messages.error(
                request,
                f'''The degreed2 enterprise customer content metadata
                “<Degreed2EnterpriseCustomerConfiguration for Enterprise
                {obj.enterprise_customer.name}>” was not updated successfully.''',
            )
        return HttpResponseRedirect('/admin/degreed2/degreed2enterprisecustomerconfiguration')
    force_content_metadata_transmission.label = "Force content metadata transmission"


@admin.register(Degreed2LearnerDataTransmissionAudit)
class Degreed2LearnerDataTransmissionAuditAdmin(BaseLearnerDataTransmissionAuditAdmin):
    """
    Django admin model for Degreed2LearnerDataTransmissionAudit.
    """
    list_display = (
        "enterprise_course_enrollment_id",
        "course_id",
        "status",
        "modified",
    )

    readonly_fields = (
        "degreed_user_email",
        "progress_status",
        "content_title",
        "enterprise_customer_name",
        "friendly_status_message",
        "api_record",
    )

    list_per_page = 1000

    class Meta:
        model = Degreed2LearnerDataTransmissionAudit



"""
Views for the ``enterprise-customer`` API endpoint.
"""

from urllib.parse import quote_plus, unquote

from edx_rbac.decorators import permission_required
from rest_framework import permissions
from rest_framework.decorators import action
from rest_framework.exceptions import ValidationError
from rest_framework.response import Response
from rest_framework.status import (
    HTTP_200_OK,
    HTTP_201_CREATED,
    HTTP_202_ACCEPTED,
    HTTP_400_BAD_REQUEST,
    HTTP_409_CONFLICT,
)

from django.contrib import auth
from django.core import exceptions
from django.db import transaction
from django.db.models import Q
from django.shortcuts import get_object_or_404
from django.utils.decorators import method_decorator

from enterprise import models
from enterprise.api.filters import EnterpriseLinkedUserFilterBackend
from enterprise.api.pagination import PaginationWithFeatureFlags
from enterprise.api.throttles import HighServiceUserThrottle
from enterprise.api.v1 import serializers
from enterprise.api.v1.decorators import require_at_least_one_query_parameter
from enterprise.api.v1.permissions import IsInEnterpriseGroup
from enterprise.api.v1.views.base_views import EnterpriseReadWriteModelViewSet
from enterprise.constants import PATHWAY_CUSTOMER_ADMIN_ENROLLMENT
from enterprise.errors import LinkUserToEnterpriseError, UnlinkUserFromEnterpriseError
from enterprise.logging import getEnterpriseLogger
from enterprise.utils import (
    enroll_subsidy_users_in_courses,
    get_best_mode_from_course_key,
    track_enrollment,
    validate_email_to_link,
)

User = auth.get_user_model()

LOGGER = getEnterpriseLogger(__name__)


class EnterpriseCustomerViewSet(EnterpriseReadWriteModelViewSet):
    """
    API views for the ``enterprise-customer`` API endpoint.
    """
    throttle_classes = (HighServiceUserThrottle, )
    queryset = models.EnterpriseCustomer.active_customers.all()
    serializer_class = serializers.EnterpriseCustomerSerializer
    filter_backends = EnterpriseReadWriteModelViewSet.filter_backends + (EnterpriseLinkedUserFilterBackend,)
    pagination_class = PaginationWithFeatureFlags

    USER_ID_FILTER = 'enterprise_customer_users__user_id'
    FIELDS = (
        'uuid', 'slug', 'name', 'active', 'site', 'enable_data_sharing_consent',
        'enforce_data_sharing_consent',
    )
    filterset_fields = FIELDS
    ordering_fields = FIELDS

    def get_permissions(self):
        if self.action == 'create':
            return [permissions.IsAuthenticated()]
        elif self.action == 'partial_update':
            return [permissions.IsAuthenticated()]
        else:
            return [permission() for permission in self.permission_classes]

    def get_serializer_class(self):
        if self.action == 'basic_list':
            return serializers.EnterpriseCustomerBasicSerializer
        return self.serializer_class

    @action(detail=False)
    # pylint: disable=unused-argument
    def basic_list(self, request, *arg, **kwargs):
        """
        Enterprise Customer's Basic data list without pagination

        Two query parameters are supported:
        - name_or_uuid: filter by name or uuid substring search in a single query parameter.
        Primarily used for frontend debounced input search.
        - startswith: filter by name starting with the given string
        """
        startswith = request.GET.get('startswith')
        name_or_uuid = request.GET.get('name_or_uuid')
        queryset = self.get_queryset().order_by('name')
        if startswith:
            queryset = queryset.filter(name__istartswith=startswith)
        if name_or_uuid:
            queryset = queryset.filter(Q(name__icontains=name_or_uuid) | Q(uuid__icontains=name_or_uuid))
        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)

    @permission_required('enterprise.can_access_admin_dashboard')
    def create(self, request, *args, **kwargs):
        """
        POST /enterprise/api/v1/enterprise-customer/
        """
        return super().create(request, *args, **kwargs)

    @permission_required('enterprise.can_access_admin_dashboard', fn=lambda request, pk: pk)
    def partial_update(self, request, *args, **kwargs):
        return super().partial_update(request, *args, **kwargs)

    @method_decorator(require_at_least_one_query_parameter('course_run_ids', 'program_uuids'))
    @action(detail=True)
    @permission_required('enterprise.can_view_catalog', fn=lambda request, pk, course_run_ids, program_uuids: pk)
    # pylint: disable=unused-argument
    def contains_content_items(self, request, pk, course_run_ids, program_uuids):
        """
        Return whether or not the specified content is available to the EnterpriseCustomer.

        Multiple course_run_ids and/or program_uuids query parameters can be sent to this view to check
        for their existence in the EnterpriseCustomerCatalogs associated with this EnterpriseCustomer.
        At least one course run key or program UUID value must be included in the request.
        """
        enterprise_customer = self.get_object()

        # Maintain plus characters in course key.
        course_run_ids = [unquote(quote_plus(course_run_id)) for course_run_id in course_run_ids]

        contains_content_items = False
        for catalog in enterprise_customer.enterprise_customer_catalogs.all():
            contains_course_runs = not course_run_ids or catalog.contains_courses(course_run_ids)
            contains_program_uuids = not program_uuids or catalog.contains_programs(program_uuids)
            if contains_course_runs and contains_program_uuids:
                contains_content_items = True
                break

        return Response({'contains_content_items': contains_content_items})

    @action(methods=['post'], permission_classes=[permissions.IsAuthenticated], detail=True)
    @permission_required('enterprise.can_enroll_learners', fn=lambda request, pk: pk)
    # pylint: disable=unused-argument
    def course_enrollments(self, request, pk):
        """
        Creates a course enrollment for an EnterpriseCustomerUser.
        """
        enterprise_customer = self.get_object()
        serializer = serializers.EnterpriseCustomerCourseEnrollmentsSerializer(
            data=request.data,
            many=True,
            context={
                'enterprise_customer': enterprise_customer,
                'request_user': request.user,
            }
        )
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=HTTP_200_OK)

        return Response(serializer.errors, status=HTTP_400_BAD_REQUEST)

    @action(detail=True, methods=['post'], permission_classes=[permissions.IsAuthenticated])
    @permission_required('enterprise.can_enroll_learners', fn=lambda request, pk: pk)
    # pylint: disable=unused-argument, too-many-statements
    def enroll_learners_in_courses(self, request, pk):
        """
        Creates a set of enterprise enrollments for specified learners by bulk enrolling them in provided courses.
        This endpoint is not transactional, in that any one or more failures will not affect other successful
        enrollments made within the same request.

        Parameters:
            enrollments_info (list of dicts): an array of dictionaries, each containing the necessary information to
                create an enrollment based on a subsidy for a user in a specified course. Each dictionary must contain
                the following keys:

                * 'user_id' OR 'email': Either unique identifier describing the user to enroll.
                * 'course_run_key': The course to enroll into.
                * 'license_uuid' OR 'transaction_id': ID of either accepted form of subsidy. `license_uuid` refers to
                  subscription licenses, and `transaction_id` refers to Learner Credit transactions.
                * 'force_enrollment' (bool, optional): Enroll even if enrollment deadline is expired (default False).

                `licenses_info` is also accepted as a body param name.

                Example::

                    enrollments_info: [
                        {
                            'email': 'newuser@test.com',
                            'course_run_key': 'course-v1:edX+DemoX+Demo_Course',
                            'license_uuid': '5b77bdbade7b4fcb838f8111b68e18ae',
                        },
                        {
                            'email': 'newuser2@test.com',
                            'course_run_key': 'course-v2:edX+FunX+Fun_Course',
                            'transaction_id': '84kdbdbade7b4fcb838f8asjke8e18ae',
                        },
                        {
                            'user_id': 1234,
                            'course_run_key': 'course-v2:edX+SadX+Sad_Course',
                            'transaction_id': 'ba1f7b61951987dc2e1743fa4886b62d',
                        },
                        ...
                    ]

            discount (int): the percent discount to be applied to all enrollments. Defaults to 100.

        Returns:
            Success cases:
                - All users exist and are enrolled -
                    {'successes': [], 'pending': [], 'failures': []}, 201
                - Some or none of the users exist but are enrolled -
                    {'successes': [], 'pending': [], 'failures': []}, 202

            Failure cases:
                - Some or all of the users can't be enrolled, no users were enrolled -
                    {'successes': [], 'pending': [], 'failures': []}, 409

                - Some or all of the provided emails are invalid
                    {'successes': [], 'pending': [], 'failures': [] 'invalid_email_addresses': []}, 409
        """
        enterprise_customer = self.get_object()
        serializer = serializers.EnterpriseCustomerBulkSubscriptionEnrollmentsSerializer(
            data=request.data,
            context={
                'enterprise_customer': enterprise_customer,
                'request_user': request.user,
            }
        )
        try:
            serializer.is_valid(raise_exception=True)
        except ValidationError:
            error_message = "Something went wrong while validating bulk enrollment requests." \
                            "Received exception: {}".format(serializer.errors)
            LOGGER.warning(error_message)
            return Response(serializer.errors, status=HTTP_400_BAD_REQUEST)

        user_id_errors = []
        email_errors = []
        serialized_data = serializer.validated_data
        enrollments_info = serialized_data.get('licenses_info', serialized_data.get('enrollments_info'))

        # Default subscription discount is 100%
        discount = serialized_data.get('discount', 100.00)

        # Retrieve and store course modes for each unique course provided
        course_runs_modes = {enrollment_info['course_run_key']: None for enrollment_info in enrollments_info}
        for course_run in course_runs_modes:
            course_runs_modes[course_run] = get_best_mode_from_course_key(course_run)

        emails = set()

        for info in enrollments_info:
            if 'user_id' in info:
                user = User.objects.filter(id=info['user_id']).first()
                if user:
                    info['email'] = user.email
                    emails.add(user.email)
                else:
                    user_id_errors.append(info['user_id'])
            else:
                emails.add(info['email'])
            info['course_mode'] = course_runs_modes[info['course_run_key']]

        for email in emails:
            try:
                validate_email_to_link(email, enterprise_customer, raise_exception=False)
            except exceptions.ValidationError:
                email_errors.append(email)

        for email in emails:
            try:
                models.EnterpriseCustomerUser.all_objects.link_user(enterprise_customer, email)
            except LinkUserToEnterpriseError:
                email_errors.append(email)

        # Remove the bad emails and bad user_ids from enrollments_info; don't attempt to enroll or link them.
        enrollments_info = [
            info for info in enrollments_info
            if info.get('email') not in email_errors and info.get('user_id') not in user_id_errors
        ]

        results = enroll_subsidy_users_in_courses(enterprise_customer, enrollments_info, discount)

        # collect the returned activation links for licenses which need activation
        activation_links = {}
        for result_kind in ['successes', 'pending']:
            for result in results[result_kind]:
                if result.get('activation_link') is not None:
                    activation_links[result['email']] = result.get('activation_link')

        for course_run in course_runs_modes:
            pending_users = {
                result.pop('user') for result in results['pending']
                if result['course_run_key'] == course_run and result.get('created')
            }
            existing_users = {
                result.pop('user') for result in results['successes']
                if result['course_run_key'] == course_run and result.get('created')
            }
            if len(pending_users | existing_users) > 0:
                LOGGER.info("Successfully bulk enrolled learners: {} into course {}".format(
                    pending_users | existing_users,
                    course_run,
                ))
                track_enrollment(PATHWAY_CUSTOMER_ADMIN_ENROLLMENT, request.user.id, course_run)
                if serializer.validated_data.get('notify'):
                    enterprise_customer.notify_enrolled_learners(
                        catalog_api_user=request.user,
                        course_id=course_run,
                        users=pending_users | existing_users,
                        admin_enrollment=True,
                        activation_links=activation_links,
                    )

        # Remove the user object from the results for any already existing enrollment cases (ie created = False) as
        # these are not JSON serializable
        existing_enrollments = []
        for result in results['pending']:
            already_enrolled_pending_user = result.pop('user', None)
            existing_enrollments.append(already_enrolled_pending_user)

        for result in results['successes']:
            already_enrolled_user = result.pop('user', None)
            existing_enrollments.append(already_enrolled_user)

        if existing_enrollments:
            LOGGER.info(
                f'Bulk enrollment request submitted for users: {existing_enrollments} who already have enrollments'
            )

        if user_id_errors:
            results['invalid_user_ids'] = user_id_errors
        if email_errors:
            results['invalid_email_addresses'] = email_errors

        if results['failures'] or email_errors or user_id_errors:
            return Response(results, status=HTTP_409_CONFLICT)
        if results['pending']:
            return Response(results, status=HTTP_202_ACCEPTED)
        return Response(results, status=HTTP_201_CREATED)

    @method_decorator(require_at_least_one_query_parameter('permissions'))
    @action(permission_classes=[permissions.IsAuthenticated, IsInEnterpriseGroup], detail=False)
    def with_access_to(self, request, *args, **kwargs):
        """
        Returns the list of enterprise customers the user has a specified group permission access to.
        """
        self.queryset = self.queryset.order_by('name')
        enterprise_id = self.request.query_params.get('enterprise_id', None)
        enterprise_slug = self.request.query_params.get('enterprise_slug', None)
        enterprise_name = self.request.query_params.get('search', None)

        if enterprise_id is not None:
            self.queryset = self.queryset.filter(uuid=enterprise_id)
        elif enterprise_slug is not None:
            self.queryset = self.queryset.filter(slug=enterprise_slug)
        elif enterprise_name is not None:
            self.queryset = self.queryset.filter(name__icontains=enterprise_name)
        return self.list(request, *args, **kwargs)

    @action(detail=False)
    @permission_required('enterprise.can_access_admin_dashboard')
    def dashboard_list(self, request, *args, **kwargs):
        """
        Supports listing dashboard enterprises for frontend-app-admin-portal.
        """
        self.queryset = self.queryset.order_by('name')
        enterprise_id = self.request.query_params.get('enterprise_id', None)
        enterprise_slug = self.request.query_params.get('enterprise_slug', None)
        enterprise_name = self.request.query_params.get('search', None)

        if enterprise_id is not None:
            self.queryset = self.queryset.filter(uuid=enterprise_id)
        elif enterprise_slug is not None:
            self.queryset = self.queryset.filter(slug=enterprise_slug)
        elif enterprise_name is not None:
            self.queryset = self.queryset.filter(name__icontains=enterprise_name)
        return self.list(request, *args, **kwargs)

    @action(methods=['patch'], detail=True, permission_classes=[permissions.IsAuthenticated])
    @permission_required('enterprise.can_access_admin_dashboard')
    def toggle_universal_link(self, request, pk=None):
        """
        Enables/Disables universal link config.
        """

        enterprise_customer = get_object_or_404(models.EnterpriseCustomer, uuid=pk)
        serializer = serializers.EnterpriseCustomerToggleUniversalLinkSerializer(
            data=request.data,
            context={
                'enterprise_customer': enterprise_customer,
                'request_user': request.user,
            }
        )

        if not serializer.is_valid():
            return Response(serializer.errors, status=HTTP_400_BAD_REQUEST)

        enable_universal_link = serializer.validated_data.get('enable_universal_link')

        if enterprise_customer.enable_universal_link == enable_universal_link:
            return Response({"detail": "No changes"}, status=HTTP_200_OK)

        enterprise_customer.toggle_universal_link(
            enable_universal_link,
        )

        response_body = {"enable_universal_link": enable_universal_link}
        headers = self.get_success_headers(response_body)
        return Response(response_body, status=HTTP_200_OK, headers=headers)

    @action(methods=['post'], detail=True, permission_classes=[permissions.IsAuthenticated])
    @permission_required('enterprise.can_access_admin_dashboard', fn=lambda request, pk: pk)
    def unlink_users(self, request, pk=None):  # pylint: disable=unused-argument
        """
        Unlinks users with the given emails from the enterprise.
        """

        serializer = serializers.EnterpriseCustomerUnlinkUsersSerializer(
            data=request.data
        )

        serializer.is_valid(raise_exception=True)

        enterprise_customer = self.get_object()
        emails_to_unlink = serializer.data.get('user_emails', [])
        is_relinkable = serializer.data.get('is_relinkable', True)

        with transaction.atomic():
            for email in emails_to_unlink:
                try:
                    models.EnterpriseCustomerUser.objects.unlink_user(
                        enterprise_customer=enterprise_customer,
                        user_email=email,
                        is_relinkable=is_relinkable
                    )
                except (models.EnterpriseCustomerUser.DoesNotExist, models.PendingEnterpriseCustomerUser.DoesNotExist):
                    msg = "User with email {} does not exist in enterprise {}.".format(email, enterprise_customer)
                    LOGGER.warning(msg)
                except Exception as exc:
                    msg = "Could not unlink {} from {}".format(email, enterprise_customer)
                    raise UnlinkUserFromEnterpriseError(msg) from exc

        return Response(status=HTTP_200_OK)



# Generated by Django 3.2.23 on 2024-04-08 07:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0203_auto_20240312_1527'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomer',
            name='enable_one_academy',
            field=models.BooleanField(default=False, help_text='If checked, search will be replaced with one academy on enterprise learner portal.', verbose_name='Enable One Academy feature'),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomer',
            name='enable_one_academy',
            field=models.BooleanField(default=False, help_text='If checked, search will be replaced with one academy on enterprise learner portal.', verbose_name='Enable One Academy feature'),
        ),
    ]



from logging import getLogger
from django.db import migrations

LOGGER = getLogger(__name__)

def update_email(apps, schema_editor):
    email_template = apps.get_model('enterprise', 'EnrollmentNotificationEmailTemplate')
    email_template.objects.update_or_create(
            template_type='ADMIN_ENROLL',
            defaults={'html_template': """
<html>
<head>
    <style type="text/css">
        a, a:visited, a:hover, a:active {
            text-decoration: none;
            color: #00262b;
        }
    </style>
</head>
<body>
    <table style="max-width: 650;" align="center">
        <tbody>
            <tr>
            <td>
                <table width="100%" cellspacing="0" cellpadding="0" role="presentation" style="border-collapse:collapse;border-spacing:0px">
                    <tbody>
                        <tr>
                        <td align="left" valign="top" style="padding:8px;Margin:0;width:560px">
                            <img src="https://ci3.googleusercontent.com/proxy/ZAml-YgIIJEht-OBl6GqLNIUeVKLyPvs_ldFo9FShp-andj1YQxvliJXv_s_Tmh6cg1-5avJJmvXmzxbQp06sB_WUeYYkN9kzV6jtVUvYKPPjUX_8_iFJAZqsNqYakl4nQyHnl0dClFzxiaLuULSPoqGPLBfJmNOEXIBYkvKYa95640xvHwDiQ22bZ16=s0-d-e1-ft#https://appboy-images.com/appboy/communication/assets/image_assets/images/5fc568622213594dcbda2623/original.png?1606772834" width="110" height="57" border="0" style="display:block" alt="edX" id="m_-37988256656304111logo" class="CToWUd">
                        </td>
                        <td align="right" valign="top" style="white-space:nowrap;text-decoration:none;color:#00262b;font-family:'inter',Arial,sans-serif;font-size:18px;line-height:21px">
                            <a href="{{dashboard_url}}">My Dashboard</a> 
                        </td>
                        </tr>
                    </tbody>
                </table>
            </td>
            </tr>
            <tr>
            <td align="left" bgcolor="#002b2b" style="Margin:0;padding-left:20px;padding-right:20px;padding-top:40px;padding-bottom:40px;background-color:#002b2b;background-image:url(https://ci3.googleusercontent.com/proxy/2CLnc9QL2u1L0MsUErVcQVBOz6OlDew2A5O8umOI9v7PGI3ip8YnJqYPRcexkjGsbNvNa6kUFyuHAMp7LlVBKKa7bxaKUjEO566AyX4M_6PhDtwz-QpLXLg9eQZQ93LIwP-5SbUtfxd203xXFTVBatJrN9P2hsuJSWFwd9k2pooiea6Qsg=s0-d-e1-ft#https://fzvpwi.stripocdn.email/content/guids/CABINET_4d3c6887b8ac137f656a3dd54bb5f6c8/images/53481614126118338.png);background-repeat:no-repeat;background-position:left top" background="https://ci3.googleusercontent.com/proxy/2CLnc9QL2u1L0MsUErVcQVBOz6OlDew2A5O8umOI9v7PGI3ip8YnJqYPRcexkjGsbNvNa6kUFyuHAMp7LlVBKKa7bxaKUjEO566AyX4M_6PhDtwz-QpLXLg9eQZQ93LIwP-5SbUtfxd203xXFTVBatJrN9P2hsuJSWFwd9k2pooiea6Qsg=s0-d-e1-ft#https://fzvpwi.stripocdn.email/content/guids/CABINET_4d3c6887b8ac137f656a3dd54bb5f6c8/images/53481614126118338.png">
                <table cellpadding="0" cellspacing="0" width="100%" style="border-collapse:collapse;border-spacing:0px">
                    <tbody>
                        <tr>
                        <td align="center" valign="top" style="padding:0;Margin:0;width:560px">
                            <table cellpadding="0" cellspacing="0" width="100%" role="presentation" style="border-collapse:collapse;border-spacing:0px">
                                <tbody>
                                    <tr>
                                    <td align="center" style="padding:0;Margin:0">
                                        <p style="Margin:0;font-size:32px;font-family:helvetica,'helvetica neue',arial,verdana,sans-serif;line-height:48px;color:#ffffff"><strong><span class="il">Congratulations</span>, Restless Learner</strong></p>
                                    </td>
                                    </tr>
                                    <tr>
                                    <td align="center" style="padding:0;Margin:0">
                                        <p style="Margin:0;font-size:20px;font-family:arial,'helvetica neue',helvetica,sans-serif;line-height:30px;color:#ffffff">Great News! You've been <span class="il">Enrolled</span> in {{enrolled_in.name}} by {{organization_name}}</p>
                                    </td>
                                    </tr>
                                </tbody>
                            </table>
                        </td>
                        </tr>
                    </tbody>
                </table>
            </td>
            </tr>
            <tr>
            <td align="left" style="padding:0;Margin:0;padding-top:20px;padding-left:20px;padding-right:20px">
                <table cellpadding="0" cellspacing="0" width="100%" style="border-collapse:collapse;border-spacing:0px">
                    <tbody>
                        <tr>
                        <td align="center" valign="top" style="padding:0;Margin:0;width:560px">
                            <table cellpadding="0" cellspacing="0" width="100%" role="presentation" style="border-collapse:collapse;border-spacing:0px">
                                <tbody>
                                    <tr>
                                    <td align="center" style="padding:0;Margin:0">
                                        <p style="Margin:0;font-size:14px;font-family:arial,'helvetica neue',helvetica,sans-serif;line-height:21px;color:#333333">
                                            This course is a free benefit offered especially for you, and we are excited for you to meet your learning community on edX.
                                        </p>
                                    </td>
                                    </tr>
                                </tbody>
                            </table>
                        </td>
                        </tr>
                    </tbody>
                </table>
            </td>
            </tr>
            <tr>
            <td>
                <table width="100%" cellspacing="0" cellpadding="0" role="presentation" style="border-collapse:collapse;border-spacing:0px">
                    <tbody>
                        <tr>
                        <td align="center" bgcolor="#ffffff" style="padding:0;Margin:0;padding-top:10px;padding-left:10px;padding-bottom:15px"><span class="m_-37988256656304111es-button-border" style="border-style:solid;border-color:#d03529;background:#d03529;border-width:0px 0px 2px 0px;display:inline-block;border-radius:0px;width:auto"><a href="{{enrolled_in.url}}" class="m_-37988256656304111es-button" style="text-decoration:none;font-family:helvetica,'helvetica neue',arial,verdana,sans-serif;font-size:18px;color:#ffffff;border-style:solid;border-color:#d03529;border-width:10px 15px;display:inline-block;background:#d03529;border-radius:0px;font-weight:normal;font-style:normal;line-height:22px;width:auto;text-align:center" target="_blank">Start my course</a></span></td>
                        </tr>
                    </tbody>
                </table>
            </td>
            </tr>
            <tr>
            <td align="center" bgcolor="#ffffff" style="padding:20px;Margin:0;background-color:#ffffff">
                <table cellpadding="0" cellspacing="0" class="m_-37988256656304111es-left" align="center" style="display:inline-block;border-collapse:collapse;border-spacing:0px;">
                    <tbody>
                        <tr>
                        <td class="m_-37988256656304111es-m-p0r" align="center" style="padding:0;Margin:0;width:50px">
                            <table cellpadding="0" cellspacing="0" width="100%" role="presentation" style="border-collapse:collapse;border-spacing:0px">
                                <tbody>
                                    <tr>
                                    <td align="center" style="padding:0;Margin:0;font-size:0px"><a href="https://www.facebook.com/edX"><img src="https://ci3.googleusercontent.com/proxy/2EBI4EepyVUk0cIRxzfMGmsBqU6TKjJsw7CqB8hSM0zRlLb0-BL0Y_JXhu-E6b1DiMGQ-4NHHem0qkjDgG_0xHNTYSsmJ0Rv9qBXCj2SJyT-yFlrkA7SYr4YkGg8CdkRYlkwZsQzU0DiZIDWZopRyOkPvcHWVTLwzCdJMKZcFc57HfR-eQ=s0-d-e1-ft#https://fzvpwi.stripocdn.email/content/guids/CABINET_4d3c6887b8ac137f656a3dd54bb5f6c8/images/71001614127047298.png" alt="facebook logo hyperlink" style="display:block;border:0;outline:none;text-decoration:none" width="35" height="35" class="CToWUd"></a></td>
                                    </tr>
                                </tbody>
                            </table>
                        </td>
                        <td class="m_-37988256656304111es-hidden" style="padding:0;Margin:0;width:15px"></td>
                        </tr>
                    </tbody>
                </table>
                <table cellpadding="0" cellspacing="0" class="m_-37988256656304111es-left" align="center" style="display:inline-block;border-collapse:collapse;border-spacing:0px;">
                    <tbody>
                        <tr>
                        <td align="center" style="padding:0;Margin:0;width:50px">
                            <table cellpadding="0" cellspacing="0" width="100%" role="presentation" style="border-collapse:collapse;border-spacing:0px">
                                <tbody>
                                    <tr>
                                    <td align="center" style="padding:0;Margin:0;font-size:0px"><a href="https://twitter.com/edXOnline"><img src="https://ci4.googleusercontent.com/proxy/yPnmVX7UE9sXw9LbZmrtTLvkrZVy9NJurDVqU6aByY871Dj0GMzEmJZOQm5kTImdMT_qNIIvpVpTpTpboYZB6wPyZuOUKwd49j6Q7yTB9IM0DWSMdldQ4XiOHReWms7Mw7HL9rRGk1iRLwmx0IT5frUGIcmc4ulwvQcjBFSHUBcMBDrAvQ=s0-d-e1-ft#https://fzvpwi.stripocdn.email/content/guids/CABINET_4d3c6887b8ac137f656a3dd54bb5f6c8/images/19131614127052508.png" alt="twitter logo hyperlink" style="display:block;border:0;outline:none;text-decoration:none" width="35" height="35" class="CToWUd"></a></td>
                                    </tr>
                                </tbody>
                            </table>
                        </td>
                        <td class="m_-37988256656304111es-hidden" style="padding:0;Margin:0;width:15px"></td>
                        </tr>
                    </tbody>
                </table>
                <table cellpadding="0" cellspacing="0" class="m_-37988256656304111es-left" align="center" style="display:inline-block;border-collapse:collapse;border-spacing:0px">
                    <tbody>
                        <tr>
                        <td align="center" style="padding:0;Margin:0;width:50px">
                            <table cellpadding="0" cellspacing="0" width="100%" role="presentation" style="border-collapse:collapse;border-spacing:0px">
                                <tbody>
                                    <tr>
                                    <td align="center" style="padding:0;Margin:0;font-size:0px"><a href="https://www.linkedin.com/school/edx/"><img src="https://ci6.googleusercontent.com/proxy/GT773Vs37lIPuUvoTpVMvtQ8sSuiC9sGLGtWGNti6nwmrZGrcewvoO_zFG4XJadUV-xPRYqf9zbLEshuVcSQMZiI1yUuM5VKxTF9lLfuzDWV1ZbMPEpk2cAWEZcDAW8mD3VFxY5l0y1Gxvcc_2bB4L-ApvPHnnIw7rXB_XDe1o37B4xPaA=s0-d-e1-ft#https://fzvpwi.stripocdn.email/content/guids/CABINET_4d3c6887b8ac137f656a3dd54bb5f6c8/images/42721614127058646.png" alt="linkedin logo hyperlink" style="display:block;border:0;outline:none;text-decoration:none" width="35" height="35" class="CToWUd"></a></td>
                                    </tr>
                                </tbody>
                            </table>
                        </td>
                        <td class="m_-37988256656304111es-hidden" style="padding:0;Margin:0;width:15px"></td>
                        </tr>
                    </tbody>
                </table>
                <table cellpadding="0" cellspacing="0" class="m_-37988256656304111es-left" align="center" style="display:inline-block;border-collapse:collapse;border-spacing:0px">
                    <tbody>
                        <tr>
                        <td align="center" style="padding:0;Margin:0;width:50px">
                            <table cellpadding="0" cellspacing="0" width="100%" role="presentation" style="border-collapse:collapse;border-spacing:0px">
                                <tbody>
                                    <tr>
                                    <td align="center" style="padding:0;Margin:0;font-size:0px"><a href="https://www.reddit.com/r/edX/"><img src="https://ci6.googleusercontent.com/proxy/hVT_30jYZguhmOyUAcTTZhCNDjjlivo3GAljGnyt_nNnjp0A0zcheoKw-XgM2NSD7M-CemzJEui6RIte_cH0YyJXUwCJWFpZ6j96QCB3T2UeQddtN1_VrVyQoDnAR1G82cHdJnq6Ysc-KQPvBKngjYaaPYSvcvMuHR0QS7dcnlGK35dUlA=s0-d-e1-ft#https://fzvpwi.stripocdn.email/content/guids/CABINET_4d3c6887b8ac137f656a3dd54bb5f6c8/images/23371614127063365.png" alt="reddit logo hyperlink" style="display:block;border:0;outline:none;text-decoration:none" width="35" height="35" class="CToWUd"></a></td>
                                    </tr>
                                </tbody>
                            </table>
                        </td>
                        <td class="m_-37988256656304111es-hidden" style="padding:0;Margin:0;width:15px"></td>
                        </tr>
                    </tbody>
                </table>
                <table cellpadding="0" cellspacing="0" class="m_-37988256656304111es-left" align="center" style="display:inline-block;border-collapse:collapse;border-spacing:0px;">
                    <tbody>
                        <tr>
                        <td align="left" style="padding:0;Margin:0;width:50px">
                            <table cellpadding="0" cellspacing="0" width="100%" role="presentation" style="border-collapse:collapse;border-spacing:0px">
                                <tbody>
                                    <tr>
                                    <td align="center" style="padding:0;Margin:0;font-size:0px"><a href="https://api.whatsapp.com/send?text=edX"><img src="https://ci4.googleusercontent.com/proxy/9UmWL708u3GDWbf_b3dzLoavIieL1kLHnipq6vGF06ZpMlLEQe9WkHJZsNVHfBwYZhpp71tbA8EJgf2_mhGV2RwDBIiUox83T4sP6uZKF3rw6QGEtQY2Ou16eR6v39Quf2AHsnh78t6JK6PMqlxGVSVKB2WkmUX37_2jtgfIqVtOLlV1bw=s0-d-e1-ft#https://fzvpwi.stripocdn.email/content/guids/CABINET_4d3c6887b8ac137f656a3dd54bb5f6c8/images/18781614127069989.png" alt="whatsapp logo hyperlink" style="display:block;border:0;outline:none;text-decoration:none" width="35" height="35" class="CToWUd"></a></td>
                                    </tr>
                                </tbody>
                            </table>
                        </td>
                        </tr>
                    </tbody>
                </table>
            </td>
            </tr>
            <tr>
            <td>
                <table cellpadding="0" cellspacing="0" width="100%">
                    <tbody>
                        <tr>
                        <td height="20" style="line-height:1px;font-size:1px"></td>
                        </tr>
                        <tr>
                        <td align="left" valign="top"><a href="https://business.edx.org" style="font-family:'Open Sans',Arial,sans-serif;font-size:14px;line-height:17px;text-decoration:none;color:#707070" target="_blank"><span style="color:#00262b">edX for Business</span><span style="color:#707070"> — eLearning Solutions for Your Company</span></a></td>
                        </tr>
                        <tr>
                        <td height="20" style="line-height:1px;font-size:1px"></td>
                        </tr>
                        <tr>
                        <td align="left" style="font-family:'Open Sans',Arial,sans-serif;color:#707070;font-size:14px;line-height:17px" valign="top">© 2021 edX Inc. All rights reserved.</td>
                        </tr>
                        <tr>
                        <td height="20" style="line-height:1px;font-size:1px"></td>
                        </tr>
                        <tr>
                        <td height="20" style="line-height:1px;font-size:1px"></td>
                        </tr>
                        <tr>
                        <td align="left" style="font-family:'Open Sans',Arial,sans-serif;color:#707070;font-size:14px;line-height:17px" valign="top">141 Portland St. 9th Floor, Cambridge, MA 02139</td>
                        </tr>
                        <tr>
                        <td height="30" style="line-height:1px;font-size:1px"></td>
                        </tr>
                    </tbody>
                </table>
            </td>
            </tr>
        </tbody>
    </table>
</body>
</html>
         """},
    )

class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0136_auto_20210629_2129'),
    ]

    operations = [
        migrations.RunPython(code=update_email, reverse_code=update_email)

    ]



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('degreed', '0004_auto_20180306_1251'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='degreedglobalconfiguration',
            name='degreed_base_url',
        ),
        migrations.RemoveField(
            model_name='degreedglobalconfiguration',
            name='degreed_user_id',
        ),
        migrations.RemoveField(
            model_name='degreedglobalconfiguration',
            name='degreed_user_password',
        ),
        migrations.RemoveField(
            model_name='degreedglobalconfiguration',
            name='provider_id',
        ),
        migrations.AddField(
            model_name='degreedenterprisecustomerconfiguration',
            name='degreed_base_url',
            field=models.CharField(help_text='The base URL used for API requests to Degreed, i.e. https://degreed.com.', max_length=255, verbose_name='Degreed Base URL', blank=True),
        ),
        migrations.AddField(
            model_name='degreedenterprisecustomerconfiguration',
            name='degreed_user_id',
            field=models.CharField(help_text='The Degreed User ID provided to the content provider by Degreed. It is required for getting the OAuth access token.', max_length=255, verbose_name='Degreed User ID', blank=True),
        ),
        migrations.AddField(
            model_name='degreedenterprisecustomerconfiguration',
            name='degreed_user_password',
            field=models.CharField(help_text='The Degreed User Password provided to the content provider by Degreed. It is required for getting the OAuth access token.', max_length=255, verbose_name='Degreed User Password', blank=True),
        ),
        migrations.AddField(
            model_name='degreedenterprisecustomerconfiguration',
            name='provider_id',
            field=models.CharField(default='EDX', help_text='The provider code that Degreed gives to the content provider.', max_length=100, verbose_name='Provider Code'),
        ),
        migrations.AddField(
            model_name='historicaldegreedenterprisecustomerconfiguration',
            name='degreed_base_url',
            field=models.CharField(help_text='The base URL used for API requests to Degreed, i.e. https://degreed.com.', max_length=255, verbose_name='Degreed Base URL', blank=True),
        ),
        migrations.AddField(
            model_name='historicaldegreedenterprisecustomerconfiguration',
            name='degreed_user_id',
            field=models.CharField(help_text='The Degreed User ID provided to the content provider by Degreed. It is required for getting the OAuth access token.', max_length=255, verbose_name='Degreed User ID', blank=True),
        ),
        migrations.AddField(
            model_name='historicaldegreedenterprisecustomerconfiguration',
            name='degreed_user_password',
            field=models.CharField(help_text='The Degreed User Password provided to the content provider by Degreed. It is required for getting the OAuth access token.', max_length=255, verbose_name='Degreed User Password', blank=True),
        ),
        migrations.AddField(
            model_name='historicaldegreedenterprisecustomerconfiguration',
            name='provider_id',
            field=models.CharField(default='EDX', help_text='The provider code that Degreed gives to the content provider.', max_length=100, verbose_name='Provider Code'),
        ),
    ]
